# Using C&W To Create Adversarial Examples







````julia
using Adversarial
using Flux
using Statistics
using Flux.Tracker: gradient, update!
using Flux.Data.MNIST
using CuArrays # comment out if a GPU is not available
using Images
using Plots
````




next we will load the trianing. For the example, we will only perform adversarial
attacks on this data

````julia
train_images = MNIST.images()
train_labels = MNIST.labels()
````


````
60000-element Array{Int64,1}:
 5
 0
 4
 1
 9
 2
 1
 3
 1
 4
 ⋮
 2
 9
 5
 1
 8
 3
 5
 6
 8
````




A function to conver the images to arrays

````julia
function minibatch(i, batch_size = 32)
    x_batch = Array{Float32}(undef, size(train_images[1])..., 1, batch_size)
    y_batch = Flux.onehotbatch(train_labels[i:i+batch_size-1], 0:9)
    for (idx, image) in enumerate(train_images[i:i+batch_size-1])
        x_batch[:,:,1,idx] = Float32.(image)
    end
    return x_batch |> gpu, y_batch |> gpu
end
````


````
minibatch (generic function with 2 methods)
````




We then create and train a simple CNN.

````julia
CNN() = Chain(
    Conv((3, 3), 1=>16, pad=(1,1), relu),
    MaxPool((2,2)),
    Conv((3, 3), 16=>32, pad=(1,1), relu),
    MaxPool((2,2)),
    Conv((3, 3), 32=>32, pad=(1,1), relu),
    MaxPool((2,2)),
    x -> reshape(x, :, size(x, 4)),
    Dense(288, 10),
    softmax) |> gpu

m = CNN();
θ = params(m);
loss(x, y) = Flux.crossentropy(m(x), y)
acc(x, y) = mean(Flux.onecold(m(x)) |> cpu .== Flux.onecold(y) |> cpu)
opt = ADAM()
````


````
ADAM(0.001, (0.9, 0.999), IdDict{Any,Any}())
````




now for our training loop...

````julia
const EPOCHS = 5
const BATCH_SIZE = 32

for epoch in 1:EPOCHS
    losses = 0.
    accuracies = 0.
    steps = 0

    for i in 1:BATCH_SIZE:size(train_images,1)-BATCH_SIZE
        x, y = minibatch(i)
        l = loss(x, y)
        a = acc(x, y)

        g = gradient(() -> l, θ)
        for p in θ
            update!(opt, p, g[p])
        end

        losses += l |> Tracker.data
        accuracies += a
        steps += 1
    end
    @info "Epoch $epoch end. Loss: $(losses / steps), Acc: $(accuracies / steps)"
end
````




Note that C&W requires the output to be the logits not a softmax function

````julia
x, y = minibatch(1, 1)
x_adv = CW(m[1:end-1], x, 9; steps = 100, c = 1.)
````


````
(f, dist(diff, x)) = (12.014841f0 (tracked), 0.00027988365f0 (tracked))
(f, dist(diff, x)) = (11.841015f0 (tracked), 0.021098785f0 (tracked))
(f, dist(diff, x)) = (11.661843f0 (tracked), 0.037883636f0 (tracked))
(f, dist(diff, x)) = (11.488434f0 (tracked), 0.0548364f0 (tracked))
(f, dist(diff, x)) = (11.315417f0 (tracked), 0.071562424f0 (tracked))
(f, dist(diff, x)) = (11.146443f0 (tracked), 0.087999046f0 (tracked))
(f, dist(diff, x)) = (10.976366f0 (tracked), 0.1043242f0 (tracked))
(f, dist(diff, x)) = (10.805944f0 (tracked), 0.12061826f0 (tracked))
(f, dist(diff, x)) = (10.634745f0 (tracked), 0.13679107f0 (tracked))
(f, dist(diff, x)) = (10.474818f0 (tracked), 0.15222117f0 (tracked))
(f, dist(diff, x)) = (10.32104f0 (tracked), 0.16762835f0 (tracked))
(f, dist(diff, x)) = (10.172176f0 (tracked), 0.18287301f0 (tracked))
(f, dist(diff, x)) = (10.032275f0 (tracked), 0.19787803f0 (tracked))
(f, dist(diff, x)) = (9.893875f0 (tracked), 0.21266712f0 (tracked))
(f, dist(diff, x)) = (9.757204f0 (tracked), 0.22735177f0 (tracked))
(f, dist(diff, x)) = (9.622189f0 (tracked), 0.2418273f0 (tracked))
(f, dist(diff, x)) = (9.487518f0 (tracked), 0.25615507f0 (tracked))
(f, dist(diff, x)) = (9.351927f0 (tracked), 0.27035853f0 (tracked))
(f, dist(diff, x)) = (9.216043f0 (tracked), 0.2845107f0 (tracked))
(f, dist(diff, x)) = (9.079821f0 (tracked), 0.2986362f0 (tracked))
(f, dist(diff, x)) = (8.9454365f0 (tracked), 0.31276593f0 (tracked))
(f, dist(diff, x)) = (8.812483f0 (tracked), 0.3268589f0 (tracked))
(f, dist(diff, x)) = (8.678783f0 (tracked), 0.34093243f0 (tracked))
(f, dist(diff, x)) = (8.545595f0 (tracked), 0.35499248f0 (tracked))
(f, dist(diff, x)) = (8.414212f0 (tracked), 0.36902377f0 (tracked))
(f, dist(diff, x)) = (8.285664f0 (tracked), 0.3830002f0 (tracked))
(f, dist(diff, x)) = (8.158448f0 (tracked), 0.39687598f0 (tracked))
(f, dist(diff, x)) = (8.030081f0 (tracked), 0.41068625f0 (tracked))
(f, dist(diff, x)) = (7.8992114f0 (tracked), 0.42441893f0 (tracked))
(f, dist(diff, x)) = (7.7671437f0 (tracked), 0.43807313f0 (tracked))
(f, dist(diff, x)) = (7.634173f0 (tracked), 0.4516614f0 (tracked))
(f, dist(diff, x)) = (7.502299f0 (tracked), 0.46512643f0 (tracked))
(f, dist(diff, x)) = (7.369721f0 (tracked), 0.47854522f0 (tracked))
(f, dist(diff, x)) = (7.2398114f0 (tracked), 0.49192145f0 (tracked))
(f, dist(diff, x)) = (7.113506f0 (tracked), 0.5051933f0 (tracked))
(f, dist(diff, x)) = (6.986439f0 (tracked), 0.518389f0 (tracked))
(f, dist(diff, x)) = (6.8588824f0 (tracked), 0.53154933f0 (tracked))
(f, dist(diff, x)) = (6.7288117f0 (tracked), 0.5446425f0 (tracked))
(f, dist(diff, x)) = (6.598542f0 (tracked), 0.5576749f0 (tracked))
(f, dist(diff, x)) = (6.466385f0 (tracked), 0.57067657f0 (tracked))
(f, dist(diff, x)) = (6.335698f0 (tracked), 0.58365387f0 (tracked))
(f, dist(diff, x)) = (6.2043004f0 (tracked), 0.5966428f0 (tracked))
(f, dist(diff, x)) = (6.074789f0 (tracked), 0.609661f0 (tracked))
(f, dist(diff, x)) = (5.9471188f0 (tracked), 0.6225615f0 (tracked))
(f, dist(diff, x)) = (5.8204517f0 (tracked), 0.6354139f0 (tracked))
(f, dist(diff, x)) = (5.6917205f0 (tracked), 0.6482648f0 (tracked))
(f, dist(diff, x)) = (5.5613494f0 (tracked), 0.6611531f0 (tracked))
(f, dist(diff, x)) = (5.4314766f0 (tracked), 0.67407733f0 (tracked))
(f, dist(diff, x)) = (5.301252f0 (tracked), 0.68703324f0 (tracked))
(f, dist(diff, x)) = (5.172281f0 (tracked), 0.69996667f0 (tracked))
(f, dist(diff, x)) = (5.0422254f0 (tracked), 0.71288794f0 (tracked))
(f, dist(diff, x)) = (4.9104395f0 (tracked), 0.7258547f0 (tracked))
(f, dist(diff, x)) = (4.7800126f0 (tracked), 0.7388716f0 (tracked))
(f, dist(diff, x)) = (4.65302f0 (tracked), 0.751924f0 (tracked))
(f, dist(diff, x)) = (4.5315146f0 (tracked), 0.7649304f0 (tracked))
(f, dist(diff, x)) = (4.4122696f0 (tracked), 0.7778549f0 (tracked))
(f, dist(diff, x)) = (4.294682f0 (tracked), 0.7906917f0 (tracked))
(f, dist(diff, x)) = (4.177544f0 (tracked), 0.8034558f0 (tracked))
(f, dist(diff, x)) = (4.061231f0 (tracked), 0.8161875f0 (tracked))
(f, dist(diff, x)) = (3.9452915f0 (tracked), 0.82891726f0 (tracked))
(f, dist(diff, x)) = (3.8290076f0 (tracked), 0.84163964f0 (tracked))
(f, dist(diff, x)) = (3.7131557f0 (tracked), 0.8543484f0 (tracked))
(f, dist(diff, x)) = (3.5989046f0 (tracked), 0.8670132f0 (tracked))
(f, dist(diff, x)) = (3.4849997f0 (tracked), 0.8796326f0 (tracked))
(f, dist(diff, x)) = (3.3709016f0 (tracked), 0.89220613f0 (tracked))
(f, dist(diff, x)) = (3.256452f0 (tracked), 0.9047488f0 (tracked))
(f, dist(diff, x)) = (3.1423426f0 (tracked), 0.91726017f0 (tracked))
(f, dist(diff, x)) = (3.0300202f0 (tracked), 0.9296923f0 (tracked))
(f, dist(diff, x)) = (2.9197865f0 (tracked), 0.942078f0 (tracked))
(f, dist(diff, x)) = (2.8098207f0 (tracked), 0.95442295f0 (tracked))
(f, dist(diff, x)) = (2.6989264f0 (tracked), 0.9667287f0 (tracked))
(f, dist(diff, x)) = (2.5881543f0 (tracked), 0.9790158f0 (tracked))
(f, dist(diff, x)) = (2.4791088f0 (tracked), 0.9912351f0 (tracked))
(f, dist(diff, x)) = (2.371358f0 (tracked), 1.0033834f0 (tracked))
(f, dist(diff, x)) = (2.264008f0 (tracked), 1.0154998f0 (tracked))
(f, dist(diff, x)) = (2.1572347f0 (tracked), 1.0275797f0 (tracked))
(f, dist(diff, x)) = (2.0515294f0 (tracked), 1.0396352f0 (tracked))
(f, dist(diff, x)) = (1.9485931f0 (tracked), 1.0516585f0 (tracked))
(f, dist(diff, x)) = (1.845181f0 (tracked), 1.0636114f0 (tracked))
(f, dist(diff, x)) = (1.7425127f0 (tracked), 1.0754977f0 (tracked))
(f, dist(diff, x)) = (1.6437268f0 (tracked), 1.0872334f0 (tracked))
(f, dist(diff, x)) = (1.5447145f0 (tracked), 1.0988435f0 (tracked))
(f, dist(diff, x)) = (1.4436455f0 (tracked), 1.110391f0 (tracked))
(f, dist(diff, x)) = (1.3416214f0 (tracked), 1.1218886f0 (tracked))
(f, dist(diff, x)) = (1.2393007f0 (tracked), 1.1333437f0 (tracked))
(f, dist(diff, x)) = (1.1372566f0 (tracked), 1.1447837f0 (tracked))
(f, dist(diff, x)) = (1.0354815f0 (tracked), 1.1562479f0 (tracked))
(f, dist(diff, x)) = (0.934875f0 (tracked), 1.1677095f0 (tracked))
(f, dist(diff, x)) = (0.8336177f0 (tracked), 1.1791869f0 (tracked))
(f, dist(diff, x)) = (0.7333002f0 (tracked), 1.1906298f0 (tracked))
(f, dist(diff, x)) = (0.6329441f0 (tracked), 1.2020226f0 (tracked))
(f, dist(diff, x)) = (0.5623317f0 (tracked), 1.2133409f0 (tracked))
(f, dist(diff, x)) = (0.4944148f0 (tracked), 1.2236216f0 (tracked))
(f, dist(diff, x)) = (0.40811157f0 (tracked), 1.2330694f0 (tracked))
(f, dist(diff, x)) = (0.30684757f0 (tracked), 1.2417969f0 (tracked))
(f, dist(diff, x)) = (0.22276974f0 (tracked), 1.2499268f0 (tracked))
(f, dist(diff, x)) = (0.15241957f0 (tracked), 1.2583523f0 (tracked))
(f, dist(diff, x)) = (0.07913065f0 (tracked), 1.2670423f0 (tracked))
(f, dist(diff, x)) = (0.0024399757f0 (tracked), 1.275952f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.285077f0 (tracked))
(f, dist(diff, x)) = (10.541569f0 (tracked), 0.15653719f0 (tracked))
(f, dist(diff, x)) = (10.358063f0 (tracked), 0.16985956f0 (tracked))
(f, dist(diff, x)) = (10.176711f0 (tracked), 0.18416068f0 (tracked))
(f, dist(diff, x)) = (9.999471f0 (tracked), 0.19894712f0 (tracked))
(f, dist(diff, x)) = (9.826427f0 (tracked), 0.21417142f0 (tracked))
(f, dist(diff, x)) = (9.656966f0 (tracked), 0.22970057f0 (tracked))
(f, dist(diff, x)) = (9.499462f0 (tracked), 0.24514481f0 (tracked))
(f, dist(diff, x)) = (9.35023f0 (tracked), 0.26015848f0 (tracked))
(f, dist(diff, x)) = (9.202301f0 (tracked), 0.27515745f0 (tracked))
(f, dist(diff, x)) = (9.054874f0 (tracked), 0.29012048f0 (tracked))
(f, dist(diff, x)) = (8.907316f0 (tracked), 0.3049699f0 (tracked))
(f, dist(diff, x)) = (8.764892f0 (tracked), 0.31976888f0 (tracked))
(f, dist(diff, x)) = (8.623613f0 (tracked), 0.334436f0 (tracked))
(f, dist(diff, x)) = (8.484878f0 (tracked), 0.34898794f0 (tracked))
(f, dist(diff, x)) = (8.344973f0 (tracked), 0.36336654f0 (tracked))
(f, dist(diff, x)) = (8.204269f0 (tracked), 0.3777024f0 (tracked))
(f, dist(diff, x)) = (8.063074f0 (tracked), 0.39201772f0 (tracked))
(f, dist(diff, x)) = (7.918975f0 (tracked), 0.40633494f0 (tracked))
(f, dist(diff, x)) = (7.7736735f0 (tracked), 0.42065984f0 (tracked))
(f, dist(diff, x)) = (7.630021f0 (tracked), 0.43498498f0 (tracked))
(f, dist(diff, x)) = (7.4876785f0 (tracked), 0.44925538f0 (tracked))
(f, dist(diff, x)) = (7.3461995f0 (tracked), 0.46341333f0 (tracked))
(f, dist(diff, x)) = (7.204457f0 (tracked), 0.47750404f0 (tracked))
(f, dist(diff, x)) = (7.063536f0 (tracked), 0.49154252f0 (tracked))
(f, dist(diff, x)) = (6.9261527f0 (tracked), 0.5054968f0 (tracked))
(f, dist(diff, x)) = (6.7936897f0 (tracked), 0.5193376f0 (tracked))
(f, dist(diff, x)) = (6.66025f0 (tracked), 0.5330902f0 (tracked))
(f, dist(diff, x)) = (6.527929f0 (tracked), 0.546724f0 (tracked))
(f, dist(diff, x)) = (6.39567f0 (tracked), 0.56024367f0 (tracked))
(f, dist(diff, x)) = (6.2637687f0 (tracked), 0.573707f0 (tracked))
(f, dist(diff, x)) = (6.1327643f0 (tracked), 0.5870991f0 (tracked))
(f, dist(diff, x)) = (6.001089f0 (tracked), 0.6004454f0 (tracked))
(f, dist(diff, x)) = (5.8705683f0 (tracked), 0.6137819f0 (tracked))
(f, dist(diff, x)) = (5.7423706f0 (tracked), 0.6270914f0 (tracked))
(f, dist(diff, x)) = (5.615539f0 (tracked), 0.6403582f0 (tracked))
(f, dist(diff, x)) = (5.4898453f0 (tracked), 0.65358937f0 (tracked))
(f, dist(diff, x)) = (5.363261f0 (tracked), 0.666768f0 (tracked))
(f, dist(diff, x)) = (5.2344184f0 (tracked), 0.67986345f0 (tracked))
(f, dist(diff, x)) = (5.106416f0 (tracked), 0.69293594f0 (tracked))
(f, dist(diff, x)) = (4.9777904f0 (tracked), 0.7060206f0 (tracked))
(f, dist(diff, x)) = (4.8490653f0 (tracked), 0.71911865f0 (tracked))
(f, dist(diff, x)) = (4.720554f0 (tracked), 0.73221004f0 (tracked))
(f, dist(diff, x)) = (4.593911f0 (tracked), 0.74526834f0 (tracked))
(f, dist(diff, x)) = (4.4688306f0 (tracked), 0.75832844f0 (tracked))
(f, dist(diff, x)) = (4.345855f0 (tracked), 0.77138036f0 (tracked))
(f, dist(diff, x)) = (4.222154f0 (tracked), 0.7844236f0 (tracked))
(f, dist(diff, x)) = (4.1012383f0 (tracked), 0.7974584f0 (tracked))
(f, dist(diff, x)) = (3.9835258f0 (tracked), 0.8104441f0 (tracked))
(f, dist(diff, x)) = (3.8659873f0 (tracked), 0.82340366f0 (tracked))
(f, dist(diff, x)) = (3.7486477f0 (tracked), 0.8362861f0 (tracked))
(f, dist(diff, x)) = (3.63584f0 (tracked), 0.8491122f0 (tracked))
(f, dist(diff, x)) = (3.524887f0 (tracked), 0.86186105f0 (tracked))
(f, dist(diff, x)) = (3.4131546f0 (tracked), 0.874539f0 (tracked))
(f, dist(diff, x)) = (3.3011956f0 (tracked), 0.8871486f0 (tracked))
(f, dist(diff, x)) = (3.190413f0 (tracked), 0.89970285f0 (tracked))
(f, dist(diff, x)) = (3.0811086f0 (tracked), 0.9121642f0 (tracked))
(f, dist(diff, x)) = (2.9718747f0 (tracked), 0.92458653f0 (tracked))
(f, dist(diff, x)) = (2.8635187f0 (tracked), 0.9369733f0 (tracked))
(f, dist(diff, x)) = (2.7559824f0 (tracked), 0.9493209f0 (tracked))
(f, dist(diff, x)) = (2.6485987f0 (tracked), 0.96162516f0 (tracked))
(f, dist(diff, x)) = (2.5415397f0 (tracked), 0.97391105f0 (tracked))
(f, dist(diff, x)) = (2.435729f0 (tracked), 0.98618263f0 (tracked))
(f, dist(diff, x)) = (2.3302622f0 (tracked), 0.9983355f0 (tracked))
(f, dist(diff, x)) = (2.2254987f0 (tracked), 1.0104586f0 (tracked))
(f, dist(diff, x)) = (2.1214705f0 (tracked), 1.0225503f0 (tracked))
(f, dist(diff, x)) = (2.0199509f0 (tracked), 1.0346034f0 (tracked))
(f, dist(diff, x)) = (1.915317f0 (tracked), 1.0465982f0 (tracked))
(f, dist(diff, x)) = (1.8097019f0 (tracked), 1.0585786f0 (tracked))
(f, dist(diff, x)) = (1.7048988f0 (tracked), 1.0705553f0 (tracked))
(f, dist(diff, x)) = (1.6019177f0 (tracked), 1.0824834f0 (tracked))
(f, dist(diff, x)) = (1.4991717f0 (tracked), 1.0943364f0 (tracked))
(f, dist(diff, x)) = (1.3974018f0 (tracked), 1.1060756f0 (tracked))
(f, dist(diff, x)) = (1.2953496f0 (tracked), 1.1177593f0 (tracked))
(f, dist(diff, x)) = (1.1930423f0 (tracked), 1.1294152f0 (tracked))
(f, dist(diff, x)) = (1.0903068f0 (tracked), 1.1410687f0 (tracked))
(f, dist(diff, x)) = (0.9892812f0 (tracked), 1.1527047f0 (tracked))
(f, dist(diff, x)) = (0.88814926f0 (tracked), 1.164296f0 (tracked))
(f, dist(diff, x)) = (0.7863722f0 (tracked), 1.1758647f0 (tracked))
(f, dist(diff, x)) = (0.6837516f0 (tracked), 1.1874207f0 (tracked))
(f, dist(diff, x)) = (0.5831928f0 (tracked), 1.198926f0 (tracked))
(f, dist(diff, x)) = (0.48526144f0 (tracked), 1.2103167f0 (tracked))
(f, dist(diff, x)) = (0.41786623f0 (tracked), 1.2216225f0 (tracked))
(f, dist(diff, x)) = (0.35175276f0 (tracked), 1.2319586f0 (tracked))
(f, dist(diff, x)) = (0.26822424f0 (tracked), 1.2414824f0 (tracked))
(f, dist(diff, x)) = (0.16953897f0 (tracked), 1.2503233f0 (tracked))
(f, dist(diff, x)) = (0.08619976f0 (tracked), 1.258589f0 (tracked))
(f, dist(diff, x)) = (0.017210484f0 (tracked), 1.2670704f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.275754f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.2833464f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.289941f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.295625f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3004783f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3045759f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.307985f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3107685f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3129838f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3146824f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3159126f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3167186f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3171394f0 (tracked))
(f, dist(diff, x)) = (10.460198f0 (tracked), 0.16449364f0 (tracked))
(f, dist(diff, x)) = (10.278772f0 (tracked), 0.17850554f0 (tracked))
(f, dist(diff, x)) = (10.100957f0 (tracked), 0.19301695f0 (tracked))
(f, dist(diff, x)) = (9.927542f0 (tracked), 0.20796682f0 (tracked))
(f, dist(diff, x)) = (9.755524f0 (tracked), 0.22307086f0 (tracked))
(f, dist(diff, x)) = (9.583783f0 (tracked), 0.23842984f0 (tracked))
(f, dist(diff, x)) = (9.425286f0 (tracked), 0.25374722f0 (tracked))
(f, dist(diff, x)) = (9.277042f0 (tracked), 0.26867497f0 (tracked))
(f, dist(diff, x)) = (9.13074f0 (tracked), 0.28359908f0 (tracked))
(f, dist(diff, x)) = (8.983759f0 (tracked), 0.29843992f0 (tracked))
(f, dist(diff, x)) = (8.835784f0 (tracked), 0.31319493f0 (tracked))
(f, dist(diff, x)) = (8.690913f0 (tracked), 0.32791433f0 (tracked))
(f, dist(diff, x)) = (8.547146f0 (tracked), 0.3424483f0 (tracked))
(f, dist(diff, x)) = (8.399559f0 (tracked), 0.35690644f0 (tracked))
(f, dist(diff, x)) = (8.256454f0 (tracked), 0.37130156f0 (tracked))
(f, dist(diff, x)) = (8.112236f0 (tracked), 0.38567832f0 (tracked))
(f, dist(diff, x)) = (7.9672694f0 (tracked), 0.40007365f0 (tracked))
(f, dist(diff, x)) = (7.8200665f0 (tracked), 0.41448513f0 (tracked))
(f, dist(diff, x)) = (7.675358f0 (tracked), 0.42891347f0 (tracked))
(f, dist(diff, x)) = (7.53064f0 (tracked), 0.4433196f0 (tracked))
(f, dist(diff, x)) = (7.387322f0 (tracked), 0.45768794f0 (tracked))
(f, dist(diff, x)) = (7.244994f0 (tracked), 0.47197983f0 (tracked))
(f, dist(diff, x)) = (7.1028476f0 (tracked), 0.48623067f0 (tracked))
(f, dist(diff, x)) = (6.960764f0 (tracked), 0.50042874f0 (tracked))
(f, dist(diff, x)) = (6.82505f0 (tracked), 0.51458097f0 (tracked))
(f, dist(diff, x)) = (6.6920147f0 (tracked), 0.528619f0 (tracked))
(f, dist(diff, x)) = (6.558722f0 (tracked), 0.5425389f0 (tracked))
(f, dist(diff, x)) = (6.4256806f0 (tracked), 0.5563553f0 (tracked))
(f, dist(diff, x)) = (6.2930903f0 (tracked), 0.57001585f0 (tracked))
(f, dist(diff, x)) = (6.1607795f0 (tracked), 0.58357406f0 (tracked))
(f, dist(diff, x)) = (6.0293136f0 (tracked), 0.597073f0 (tracked))
(f, dist(diff, x)) = (5.8995495f0 (tracked), 0.6105237f0 (tracked))
(f, dist(diff, x)) = (5.7708898f0 (tracked), 0.6239229f0 (tracked))
(f, dist(diff, x)) = (5.643057f0 (tracked), 0.63727283f0 (tracked))
(f, dist(diff, x)) = (5.5137215f0 (tracked), 0.6505842f0 (tracked))
(f, dist(diff, x)) = (5.383183f0 (tracked), 0.6638874f0 (tracked))
(f, dist(diff, x)) = (5.2526217f0 (tracked), 0.67719597f0 (tracked))
(f, dist(diff, x)) = (5.1225696f0 (tracked), 0.69045764f0 (tracked))
(f, dist(diff, x)) = (4.992829f0 (tracked), 0.70369583f0 (tracked))
(f, dist(diff, x)) = (4.862557f0 (tracked), 0.71696424f0 (tracked))
(f, dist(diff, x)) = (4.734556f0 (tracked), 0.7302535f0 (tracked))
(f, dist(diff, x)) = (4.606796f0 (tracked), 0.7435292f0 (tracked))
(f, dist(diff, x)) = (4.4788156f0 (tracked), 0.7567793f0 (tracked))
(f, dist(diff, x)) = (4.3525367f0 (tracked), 0.77000535f0 (tracked))
(f, dist(diff, x)) = (4.226909f0 (tracked), 0.7831795f0 (tracked))
(f, dist(diff, x)) = (4.1016307f0 (tracked), 0.79632884f0 (tracked))
(f, dist(diff, x)) = (3.9804258f0 (tracked), 0.80946183f0 (tracked))
(f, dist(diff, x)) = (3.8612676f0 (tracked), 0.8225271f0 (tracked))
(f, dist(diff, x)) = (3.7422724f0 (tracked), 0.83554083f0 (tracked))
(f, dist(diff, x)) = (3.6266284f0 (tracked), 0.8484979f0 (tracked))
(f, dist(diff, x)) = (3.513413f0 (tracked), 0.8613908f0 (tracked))
(f, dist(diff, x)) = (3.4007401f0 (tracked), 0.8742174f0 (tracked))
(f, dist(diff, x)) = (3.2892628f0 (tracked), 0.8869676f0 (tracked))
(f, dist(diff, x)) = (3.1789584f0 (tracked), 0.8996396f0 (tracked))
(f, dist(diff, x)) = (3.0694847f0 (tracked), 0.9122205f0 (tracked))
(f, dist(diff, x)) = (2.961372f0 (tracked), 0.92470086f0 (tracked))
(f, dist(diff, x)) = (2.8536248f0 (tracked), 0.9371401f0 (tracked))
(f, dist(diff, x)) = (2.7462587f0 (tracked), 0.9495521f0 (tracked))
(f, dist(diff, x)) = (2.6392574f0 (tracked), 0.961926f0 (tracked))
(f, dist(diff, x)) = (2.5328684f0 (tracked), 0.97426194f0 (tracked))
(f, dist(diff, x)) = (2.4280682f0 (tracked), 0.98653793f0 (tracked))
(f, dist(diff, x)) = (2.3232574f0 (tracked), 0.9987501f0 (tracked))
(f, dist(diff, x)) = (2.2174478f0 (tracked), 1.0108532f0 (tracked))
(f, dist(diff, x)) = (2.112259f0 (tracked), 1.0228815f0 (tracked))
(f, dist(diff, x)) = (2.0086246f0 (tracked), 1.0348246f0 (tracked))
(f, dist(diff, x)) = (1.9045253f0 (tracked), 1.0467006f0 (tracked))
(f, dist(diff, x)) = (1.7998071f0 (tracked), 1.0585346f0 (tracked))
(f, dist(diff, x)) = (1.6941509f0 (tracked), 1.0703728f0 (tracked))
(f, dist(diff, x)) = (1.5890074f0 (tracked), 1.0822271f0 (tracked))
(f, dist(diff, x)) = (1.4839807f0 (tracked), 1.0940572f0 (tracked))
(f, dist(diff, x)) = (1.3799505f0 (tracked), 1.1058457f0 (tracked))
(f, dist(diff, x)) = (1.2769675f0 (tracked), 1.1175395f0 (tracked))
(f, dist(diff, x)) = (1.1747327f0 (tracked), 1.1291854f0 (tracked))
(f, dist(diff, x)) = (1.0728865f0 (tracked), 1.1407782f0 (tracked))
(f, dist(diff, x)) = (0.97093964f0 (tracked), 1.1523359f0 (tracked))
(f, dist(diff, x)) = (0.87079096f0 (tracked), 1.163838f0 (tracked))
(f, dist(diff, x)) = (0.77062654f0 (tracked), 1.1753038f0 (tracked))
(f, dist(diff, x)) = (0.67032766f0 (tracked), 1.1867357f0 (tracked))
(f, dist(diff, x)) = (0.57202387f0 (tracked), 1.1981387f0 (tracked))
(f, dist(diff, x)) = (0.47594213f0 (tracked), 1.209402f0 (tracked))
(f, dist(diff, x)) = (0.38049793f0 (tracked), 1.2205905f0 (tracked))
(f, dist(diff, x)) = (0.28479528f0 (tracked), 1.231707f0 (tracked))
(f, dist(diff, x)) = (0.23031998f0 (tracked), 1.2427772f0 (tracked))
(f, dist(diff, x)) = (0.16592836f0 (tracked), 1.2529914f0 (tracked))
(f, dist(diff, x)) = (0.08551836f0 (tracked), 1.2624699f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.2713f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.2790331f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.2857627f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.2915756f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.2965525f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3007678f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3042892f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.30718f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3094977f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3112944f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3126186f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.313514f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3140199f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3141735f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3140074f0 (tracked))
(f, dist(diff, x)) = (10.423694f0 (tracked), 0.16808258f0 (tracked))
(f, dist(diff, x)) = (10.242366f0 (tracked), 0.18198241f0 (tracked))
(f, dist(diff, x)) = (10.0640545f0 (tracked), 0.19657148f0 (tracked))
(f, dist(diff, x)) = (9.88981f0 (tracked), 0.21158147f0 (tracked))
(f, dist(diff, x)) = (9.717086f0 (tracked), 0.22677135f0 (tracked))
(f, dist(diff, x)) = (9.545709f0 (tracked), 0.2422335f0 (tracked))
(f, dist(diff, x)) = (9.387779f0 (tracked), 0.25754604f0 (tracked))
(f, dist(diff, x)) = (9.239054f0 (tracked), 0.27247006f0 (tracked))
(f, dist(diff, x)) = (9.090999f0 (tracked), 0.28735182f0 (tracked))
(f, dist(diff, x)) = (8.944315f0 (tracked), 0.30217338f0 (tracked))
(f, dist(diff, x)) = (8.797043f0 (tracked), 0.3169494f0 (tracked))
(f, dist(diff, x)) = (8.652798f0 (tracked), 0.33168265f0 (tracked))
(f, dist(diff, x)) = (8.509857f0 (tracked), 0.34622678f0 (tracked))
(f, dist(diff, x)) = (8.365324f0 (tracked), 0.36070645f0 (tracked))
(f, dist(diff, x)) = (8.22234f0 (tracked), 0.3751144f0 (tracked))
(f, dist(diff, x)) = (8.078768f0 (tracked), 0.38949192f0 (tracked))
(f, dist(diff, x)) = (7.933382f0 (tracked), 0.40387484f0 (tracked))
(f, dist(diff, x)) = (7.786897f0 (tracked), 0.41827503f0 (tracked))
(f, dist(diff, x)) = (7.6412334f0 (tracked), 0.43266568f0 (tracked))
(f, dist(diff, x)) = (7.497309f0 (tracked), 0.44705755f0 (tracked))
(f, dist(diff, x)) = (7.3547173f0 (tracked), 0.46142852f0 (tracked))
(f, dist(diff, x)) = (7.212349f0 (tracked), 0.4757194f0 (tracked))
(f, dist(diff, x)) = (7.0710893f0 (tracked), 0.48996678f0 (tracked))
(f, dist(diff, x)) = (6.93022f0 (tracked), 0.5041615f0 (tracked))
(f, dist(diff, x)) = (6.7916956f0 (tracked), 0.51827073f0 (tracked))
(f, dist(diff, x)) = (6.6594486f0 (tracked), 0.5322559f0 (tracked))
(f, dist(diff, x)) = (6.5263796f0 (tracked), 0.5461286f0 (tracked))
(f, dist(diff, x)) = (6.39421f0 (tracked), 0.5599005f0 (tracked))
(f, dist(diff, x)) = (6.2622733f0 (tracked), 0.57355285f0 (tracked))
(f, dist(diff, x)) = (6.1304145f0 (tracked), 0.5871474f0 (tracked))
(f, dist(diff, x)) = (5.9994164f0 (tracked), 0.60067403f0 (tracked))
(f, dist(diff, x)) = (5.868933f0 (tracked), 0.614141f0 (tracked))
(f, dist(diff, x)) = (5.7404776f0 (tracked), 0.62756914f0 (tracked))
(f, dist(diff, x)) = (5.6123204f0 (tracked), 0.640944f0 (tracked))
(f, dist(diff, x)) = (5.482728f0 (tracked), 0.6542981f0 (tracked))
(f, dist(diff, x)) = (5.3518095f0 (tracked), 0.6676422f0 (tracked))
(f, dist(diff, x)) = (5.2221155f0 (tracked), 0.6809614f0 (tracked))
(f, dist(diff, x)) = (5.0922303f0 (tracked), 0.6942416f0 (tracked))
(f, dist(diff, x)) = (4.9615326f0 (tracked), 0.70749396f0 (tracked))
(f, dist(diff, x)) = (4.8312435f0 (tracked), 0.7207837f0 (tracked))
(f, dist(diff, x)) = (4.7021375f0 (tracked), 0.73408264f0 (tracked))
(f, dist(diff, x)) = (4.573504f0 (tracked), 0.74736595f0 (tracked))
(f, dist(diff, x)) = (4.444457f0 (tracked), 0.7606095f0 (tracked))
(f, dist(diff, x)) = (4.317697f0 (tracked), 0.7737804f0 (tracked))
(f, dist(diff, x)) = (4.19065f0 (tracked), 0.786919f0 (tracked))
(f, dist(diff, x)) = (4.064255f0 (tracked), 0.80004126f0 (tracked))
(f, dist(diff, x)) = (3.9441857f0 (tracked), 0.8130811f0 (tracked))
(f, dist(diff, x)) = (3.8252435f0 (tracked), 0.82608163f0 (tracked))
(f, dist(diff, x)) = (3.7064605f0 (tracked), 0.8390472f0 (tracked))
(f, dist(diff, x)) = (3.5910144f0 (tracked), 0.8519619f0 (tracked))
(f, dist(diff, x)) = (3.4783282f0 (tracked), 0.86482114f0 (tracked))
(f, dist(diff, x)) = (3.3669472f0 (tracked), 0.87759775f0 (tracked))
(f, dist(diff, x)) = (3.2556324f0 (tracked), 0.8902929f0 (tracked))
(f, dist(diff, x)) = (3.145f0 (tracked), 0.90290517f0 (tracked))
(f, dist(diff, x)) = (3.0355096f0 (tracked), 0.91538936f0 (tracked))
(f, dist(diff, x)) = (2.926445f0 (tracked), 0.9277997f0 (tracked))
(f, dist(diff, x)) = (2.818153f0 (tracked), 0.9401497f0 (tracked))
(f, dist(diff, x)) = (2.711269f0 (tracked), 0.9524488f0 (tracked))
(f, dist(diff, x)) = (2.6048784f0 (tracked), 0.964691f0 (tracked))
(f, dist(diff, x)) = (2.498519f0 (tracked), 0.9768901f0 (tracked))
(f, dist(diff, x)) = (2.3929534f0 (tracked), 0.9890754f0 (tracked))
(f, dist(diff, x)) = (2.2874904f0 (tracked), 1.0012269f0 (tracked))
(f, dist(diff, x)) = (2.1820216f0 (tracked), 1.0133482f0 (tracked))
(f, dist(diff, x)) = (2.0769935f0 (tracked), 1.0254376f0 (tracked))
(f, dist(diff, x)) = (1.9730358f0 (tracked), 1.0374535f0 (tracked))
(f, dist(diff, x)) = (1.86836f0 (tracked), 1.0494156f0 (tracked))
(f, dist(diff, x)) = (1.7643228f0 (tracked), 1.0612924f0 (tracked))
(f, dist(diff, x)) = (1.6594129f0 (tracked), 1.0731455f0 (tracked))
(f, dist(diff, x)) = (1.5557761f0 (tracked), 1.085023f0 (tracked))
(f, dist(diff, x)) = (1.4526372f0 (tracked), 1.0968628f0 (tracked))
(f, dist(diff, x)) = (1.3494935f0 (tracked), 1.1086272f0 (tracked))
(f, dist(diff, x)) = (1.2468362f0 (tracked), 1.1203631f0 (tracked))
(f, dist(diff, x)) = (1.1455193f0 (tracked), 1.1320546f0 (tracked))
(f, dist(diff, x)) = (1.0441036f0 (tracked), 1.1437051f0 (tracked))
(f, dist(diff, x)) = (0.94354486f0 (tracked), 1.1552948f0 (tracked))
(f, dist(diff, x)) = (0.8439374f0 (tracked), 1.1668215f0 (tracked))
(f, dist(diff, x)) = (0.74346876f0 (tracked), 1.1783073f0 (tracked))
(f, dist(diff, x)) = (0.64326334f0 (tracked), 1.1897783f0 (tracked))
(f, dist(diff, x)) = (0.54425955f0 (tracked), 1.2012187f0 (tracked))
(f, dist(diff, x)) = (0.44977522f0 (tracked), 1.212557f0 (tracked))
(f, dist(diff, x)) = (0.35507393f0 (tracked), 1.2238152f0 (tracked))
(f, dist(diff, x)) = (0.25991106f0 (tracked), 1.2349875f0 (tracked))
(f, dist(diff, x)) = (0.2159853f0 (tracked), 1.2460784f0 (tracked))
(f, dist(diff, x)) = (0.1522851f0 (tracked), 1.2563043f0 (tracked))
(f, dist(diff, x)) = (0.07201958f0 (tracked), 1.2657852f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.2746129f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.2823386f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.2890551f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.2948523f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.2998097f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3040022f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3074994f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3103639f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3126543f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3144226f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3157177f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3165836f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3170604f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3171844f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3169895f0 (tracked))
(f, dist(diff, x)) = (10.453038f0 (tracked), 0.1650507f0 (tracked))
(f, dist(diff, x)) = (10.271598f0 (tracked), 0.17913364f0 (tracked))
(f, dist(diff, x)) = (10.093268f0 (tracked), 0.19375385f0 (tracked))
(f, dist(diff, x)) = (9.918779f0 (tracked), 0.20870677f0 (tracked))
(f, dist(diff, x)) = (9.746176f0 (tracked), 0.22396782f0 (tracked))
(f, dist(diff, x)) = (9.574669f0 (tracked), 0.2394908f0 (tracked))
(f, dist(diff, x)) = (9.416885f0 (tracked), 0.25488546f0 (tracked))
(f, dist(diff, x)) = (9.26765f0 (tracked), 0.26985815f0 (tracked))
(f, dist(diff, x)) = (9.119125f0 (tracked), 0.284787f0 (tracked))
(f, dist(diff, x)) = (8.972593f0 (tracked), 0.29970926f0 (tracked))
(f, dist(diff, x)) = (8.824345f0 (tracked), 0.31456417f0 (tracked))
(f, dist(diff, x)) = (8.681278f0 (tracked), 0.3293568f0 (tracked))
(f, dist(diff, x)) = (8.53821f0 (tracked), 0.3439342f0 (tracked))
(f, dist(diff, x)) = (8.39167f0 (tracked), 0.35842672f0 (tracked))
(f, dist(diff, x)) = (8.2494755f0 (tracked), 0.37286237f0 (tracked))
(f, dist(diff, x)) = (8.105162f0 (tracked), 0.38728496f0 (tracked))
(f, dist(diff, x)) = (7.9598966f0 (tracked), 0.4016947f0 (tracked))
(f, dist(diff, x)) = (7.8134046f0 (tracked), 0.4161102f0 (tracked))
(f, dist(diff, x)) = (7.667218f0 (tracked), 0.430533f0 (tracked))
(f, dist(diff, x)) = (7.52268f0 (tracked), 0.4449474f0 (tracked))
(f, dist(diff, x)) = (7.3799696f0 (tracked), 0.4593494f0 (tracked))
(f, dist(diff, x)) = (7.2373557f0 (tracked), 0.4736767f0 (tracked))
(f, dist(diff, x)) = (7.095406f0 (tracked), 0.48795956f0 (tracked))
(f, dist(diff, x)) = (6.954492f0 (tracked), 0.5021855f0 (tracked))
(f, dist(diff, x)) = (6.817375f0 (tracked), 0.51633704f0 (tracked))
(f, dist(diff, x)) = (6.684861f0 (tracked), 0.5303608f0 (tracked))
(f, dist(diff, x)) = (6.551723f0 (tracked), 0.5442593f0 (tracked))
(f, dist(diff, x)) = (6.4184575f0 (tracked), 0.558075f0 (tracked))
(f, dist(diff, x)) = (6.2857733f0 (tracked), 0.5717668f0 (tracked))
(f, dist(diff, x)) = (6.1534934f0 (tracked), 0.5853727f0 (tracked))
(f, dist(diff, x)) = (6.021498f0 (tracked), 0.59890556f0 (tracked))
(f, dist(diff, x)) = (5.8899684f0 (tracked), 0.61239016f0 (tracked))
(f, dist(diff, x)) = (5.7601027f0 (tracked), 0.6258475f0 (tracked))
(f, dist(diff, x)) = (5.6318574f0 (tracked), 0.63926685f0 (tracked))
(f, dist(diff, x)) = (5.5022497f0 (tracked), 0.65264386f0 (tracked))
(f, dist(diff, x)) = (5.3707576f0 (tracked), 0.6660115f0 (tracked))
(f, dist(diff, x)) = (5.240767f0 (tracked), 0.67937726f0 (tracked))
(f, dist(diff, x)) = (5.1108046f0 (tracked), 0.6927026f0 (tracked))
(f, dist(diff, x)) = (4.9797473f0 (tracked), 0.70600474f0 (tracked))
(f, dist(diff, x)) = (4.849834f0 (tracked), 0.7193259f0 (tracked))
(f, dist(diff, x)) = (4.720357f0 (tracked), 0.7326427f0 (tracked))
(f, dist(diff, x)) = (4.5912824f0 (tracked), 0.7459547f0 (tracked))
(f, dist(diff, x)) = (4.461477f0 (tracked), 0.7592244f0 (tracked))
(f, dist(diff, x)) = (4.3326225f0 (tracked), 0.7724588f0 (tracked))
(f, dist(diff, x)) = (4.2059293f0 (tracked), 0.7856694f0 (tracked))
(f, dist(diff, x)) = (4.079167f0 (tracked), 0.79886013f0 (tracked))
(f, dist(diff, x)) = (3.9567237f0 (tracked), 0.8120142f0 (tracked))
(f, dist(diff, x)) = (3.8364244f0 (tracked), 0.8250838f0 (tracked))
(f, dist(diff, x)) = (3.718308f0 (tracked), 0.838087f0 (tracked))
(f, dist(diff, x)) = (3.6029105f0 (tracked), 0.85102683f0 (tracked))
(f, dist(diff, x)) = (3.4898043f0 (tracked), 0.86390024f0 (tracked))
(f, dist(diff, x)) = (3.3769264f0 (tracked), 0.87671924f0 (tracked))
(f, dist(diff, x)) = (3.2638617f0 (tracked), 0.8894964f0 (tracked))
(f, dist(diff, x)) = (3.151814f0 (tracked), 0.902236f0 (tracked))
(f, dist(diff, x)) = (3.0413609f0 (tracked), 0.9148543f0 (tracked))
(f, dist(diff, x)) = (2.9317412f0 (tracked), 0.92737526f0 (tracked))
(f, dist(diff, x)) = (2.8232508f0 (tracked), 0.93983316f0 (tracked))
(f, dist(diff, x)) = (2.7153182f0 (tracked), 0.95223397f0 (tracked))
(f, dist(diff, x)) = (2.6090531f0 (tracked), 0.9645863f0 (tracked))
(f, dist(diff, x)) = (2.5026307f0 (tracked), 0.97691107f0 (tracked))
(f, dist(diff, x)) = (2.3967972f0 (tracked), 0.98920816f0 (tracked))
(f, dist(diff, x)) = (2.2907028f0 (tracked), 1.001456f0 (tracked))
(f, dist(diff, x)) = (2.185902f0 (tracked), 1.0136621f0 (tracked))
(f, dist(diff, x)) = (2.0824847f0 (tracked), 1.025813f0 (tracked))
(f, dist(diff, x)) = (1.9771152f0 (tracked), 1.0379301f0 (tracked))
(f, dist(diff, x)) = (1.8718681f0 (tracked), 1.05f0 (tracked))
(f, dist(diff, x)) = (1.766469f0 (tracked), 1.0620308f0 (tracked))
(f, dist(diff, x)) = (1.6602225f0 (tracked), 1.0739944f0 (tracked))
(f, dist(diff, x)) = (1.5554004f0 (tracked), 1.0859009f0 (tracked))
(f, dist(diff, x)) = (1.4514108f0 (tracked), 1.0977426f0 (tracked))
(f, dist(diff, x)) = (1.3482242f0 (tracked), 1.1094842f0 (tracked))
(f, dist(diff, x)) = (1.245121f0 (tracked), 1.1211829f0 (tracked))
(f, dist(diff, x)) = (1.1426225f0 (tracked), 1.1328511f0 (tracked))
(f, dist(diff, x)) = (1.0403352f0 (tracked), 1.1444907f0 (tracked))
(f, dist(diff, x)) = (0.9383521f0 (tracked), 1.1561158f0 (tracked))
(f, dist(diff, x)) = (0.8384762f0 (tracked), 1.1676883f0 (tracked))
(f, dist(diff, x)) = (0.73790836f0 (tracked), 1.1792231f0 (tracked))
(f, dist(diff, x)) = (0.6373086f0 (tracked), 1.1907356f0 (tracked))
(f, dist(diff, x)) = (0.5393276f0 (tracked), 1.2021921f0 (tracked))
(f, dist(diff, x)) = (0.44408846f0 (tracked), 1.2135211f0 (tracked))
(f, dist(diff, x)) = (0.34921837f0 (tracked), 1.2247478f0 (tracked))
(f, dist(diff, x)) = (0.25506258f0 (tracked), 1.2358747f0 (tracked))
(f, dist(diff, x)) = (0.17675686f0 (tracked), 1.2469375f0 (tracked))
(f, dist(diff, x)) = (0.11230564f0 (tracked), 1.2571328f0 (tracked))
(f, dist(diff, x)) = (0.031882763f0 (tracked), 1.2665844f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.2753836f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.2830846f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.2897829f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.2955643f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3005106f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3046957f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3081888f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3110517f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3133432f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3151153f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.316416f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3172886f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3177744f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3179088f0 (tracked))
(f, dist(diff, x)) = (0.0f0 (tracked), 1.3177248f0 (tracked))
28×28×1×1 CuArray{Float32,4}:
[:, :, 1, 1] =
 0.0         0.0        0.0       …  0.0956909   0.0895732  0.0      
 0.0         0.0        0.0          0.00936568  0.0        0.0386969
 0.0         0.0        0.0          0.0         0.0715643  0.0      
 0.0         0.0        0.0          0.0946199   0.0        0.0      
 0.0958431   0.0        0.0          0.0179298   0.0618175  0.0      
 0.108635    0.110679   0.0       …  0.0830987   0.0        0.0934809
 0.0         0.0        0.0          0.0752861   0.0        0.0      
 0.0         0.0        0.0          0.0         0.0        0.0      
 0.0         0.0961016  0.0          0.0         0.0        0.0      
 0.0         0.0        0.0          0.0         0.0        0.0      
 ⋮                                ⋱  ⋮                               
 0.0         0.0        0.0          0.0         0.0857638  0.0      
 0.0         0.0        0.0       …  0.0         0.0        0.0      
 0.0         0.0        0.0          0.0666382   0.0        0.0      
 0.0         0.0        0.0          0.0         0.0        0.0      
 0.0         0.0        0.0          0.0         0.0777554  0.0927111
 0.00351883  0.0        0.0          0.0         0.0        0.0      
 0.0         0.0937531  0.107995  …  0.0         0.0        0.0319592
 0.0         0.0        0.0          0.0         0.0        0.0      
 0.0         0.0        0.0          0.0         0.0        0.0903917
````




we can see that the predicted labels are different

````julia
adversarial_pred = m(x_adv) |> Flux.onecold |> getindex
original_pred = m(x) |> Flux.onecold |> getindex
````


````
6
````




and visualise the resulting adversarial in comparison to the original image.
When using an ϵ value of 0.07, the different is very slight, if noticable
at all.

````julia
l = @layout [a b]
adv = heatmap(permutedims(x_adv, (4, 3, 1, 2))[1,1,:,:] |> cpu)
org = heatmap(permutedims(x, (4, 3, 1, 2))[1,1,:,:] |> cpu)
plot(org, adv, layout = l)

@assert adversarial_pred != original_pred
````


